@@ -80,6 +80,7 @@ struct _MctApplication
   GtkLabel *error_message;
   GtkLockButton *lock_button;
   GtkButton *user_accounts_panel_button;
+  GtkLabel *help_label;
 };
 
 G_DEFINE_TYPE (MctApplication, mct_application, GTK_TYPE_APPLICATION)
@@ -94,9 +95,25 @@ static void
 mct_application_constructed (GObject *object)
 {
   GApplication *application = G_APPLICATION (object);
+  const GOptionEntry options[] =
+    {
+      { "user", 'u', G_OPTION_FLAG_NONE, G_OPTION_ARG_STRING, NULL,
+        /* Translators: This documents the --user command line option to malcontent-control: */
+        N_("User to select in the UI"),
+        /* Translators: This is a placeholder for a command line argument value: */
+        N_("USERNAME") },
+    };
 
   g_application_set_application_id (application, "org.freedesktop.MalcontentControl");
 
+  g_application_add_main_option_entries (application, options);
+  g_application_set_flags (application, g_application_get_flags (application) | G_APPLICATION_HANDLES_COMMAND_LINE);
+
+  /* Translators: This is a summary of what the application does, displayed when
+   * it’s run with --help: */
+  g_application_set_option_context_parameter_string (application,
+                                                     N_("— view and edit parental controls"));
+
   /* Localisation */
   bindtextdomain ("malcontent", PACKAGE_LOCALE_DIR);
   bind_textdomain_codeset ("malcontent", "UTF-8");
@@ -195,6 +212,7 @@ mct_application_activate (GApplication *application)
       self->error_message = GTK_LABEL (gtk_builder_get_object (builder, "error_message"));
       self->lock_button = GTK_LOCK_BUTTON (gtk_builder_get_object (builder, "lock_button"));
       self->user_accounts_panel_button = GTK_BUTTON (gtk_builder_get_object (builder, "user_accounts_panel_button"));
+      self->help_label = GTK_LABEL (gtk_builder_get_object (builder, "help_label"));
 
       /* Connect signals. */
       g_signal_connect_object (self->user_selector, "notify::user",
@@ -240,6 +258,25 @@ mct_application_startup (GApplication *application)
                                          "app.quit", (const gchar * const[]) { "<Primary>q", "<Primary>w", NULL });
 }
 
+static gint
+mct_application_command_line (GApplication            *application,
+                              GApplicationCommandLine *command_line)
+{
+  MctApplication *self = MCT_APPLICATION (application);
+  GVariantDict *options = g_application_command_line_get_options_dict (command_line);
+  const gchar *username;
+
+  /* Show the application. */
+  g_application_activate (application);
+
+  /* Select a user if requested. */
+  if (g_variant_dict_lookup (options, "user", "&s", &username) &&
+      !mct_user_selector_select_user_by_username (self->user_selector, username))
+    g_warning ("Failed to select user ‘%s’", username);
+
+  return 0;  /* exit status */
+}
+
 static void
 mct_application_class_init (MctApplicationClass *klass)
 {
@@ -251,6 +288,7 @@ mct_application_class_init (MctApplicationClass *klass)
 
   application_class->activate = mct_application_activate;
   application_class->startup = mct_application_startup;
+  application_class->command_line = mct_application_command_line;
 }
 
 static void
@@ -355,6 +393,24 @@ update_main_stack (MctApplication *self)
     }
   else if (is_permission_loaded && is_user_manager_loaded)
     {
+      g_autofree gchar *help_label = NULL;
+
+      /* Translators: Replace the link to commonsensemedia.org with some
+       * localised guidance for parents/carers on how to set restrictions on
+       * their child/caree in a responsible way which is in keeping with the
+       * best practice and culture of the region. If no suitable localised
+       * guidance exists, and if the default commonsensemedia.org link is not
+       * suitable, please file an issue against malcontent so we can discuss
+       * further!
+       * https://gitlab.freedesktop.org/pwithnall/malcontent/-/issues/new
+       */
+      help_label = g_strdup_printf (_("It’s recommended that restrictions are "
+                                      "set as part of an ongoing conversation "
+                                      "with %s. <a href='https://www.commonsensemedia.org/privacy-and-internet-safety'>"
+                                      "Read guidance</a> on what to consider."),
+                                    act_user_get_real_name (selected_user));
+      gtk_label_set_markup (self->help_label, help_label);
+
       mct_user_controls_set_user (self->user_controls, selected_user);
 
       new_page_name = "controls";
